//#############################################################################
//
// FILE:   DRV8329A_DRV8329C_Sensored_Trapezoidal.c
//
// TITLE:  DRV8329AEVM Sensored Trapezoidal Firmware
//

//! This example sets up the F280049C Launchpad with the DRV8329AEVM.
//!
//! \b External \b Connections \n
//!  - Remove JP1, JP2, and JP3 headers on F280049C Launchpad
//!  - Connect the DRV8329AEVM boosterpack to the upper F280049C
//!    Launchpad pins
//!  - Supply PVDD power via connector to PVDD/GND
//!
//! \b Watch \b Variables \n
//!  - OutputEnable
//!  - PwmDutyCycle
//!  - PotEnable
//!  - MotorStop
//!  - direction_CCW
//!
//
//###########################################################################
// $TI Release: F28004x Support Library v1.08.00.00 $
// $Release Date: Mon Dec 23 17:24:05 IST 2019 $
// $Copyright:
// Copyright (C) 2021 Texas Instruments Incorporated - http://www.ti.com/
//
// Redistribution and use in source and binary forms, with or without 
// modification, are permitted provided that the following conditions 
// are met:
// 
//   Redistributions of source code must retain the above copyright 
//   notice, this list of conditions and the following disclaimer.
// 
//   Redistributions in binary form must reproduce the above copyright
//   notice, this list of conditions and the following disclaimer in the 
//   documentation and/or other materials provided with the   
//   distribution.
// 
//   Neither the name of Texas Instruments Incorporated nor the names of
//   its contributors may be used to endorse or promote products derived
//   from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// $
//###########################################################################

//
// Included Files
//
#include "F28x_Project.h"     // Device Header file and Examples Include File
#include "SFO_V8.h"

//
// Defines
//

// Period register

#define nFAULT 25
#define nSLEEP 30
#define HALLA 58
#define HALLB 23
#define HALLC 59
#define MCU_LED 57
#define DRVOFF 39

#define nFAULT_flag GpioDataRegs.GPADAT.bit.GPIO25

//
// Typedef
//
typedef struct
{
    volatile struct EPWM_REGS *EPwmRegHandle;
} EPWM_INFO;

//
// Globals
//

Uint32 PWM_FREQ = 20000, PWM_FREQ_old, PWM_PERIOD;
Uint16 DEAD_TIME_ns = 0, DEAD_TIME_old;
float pwmDutyCycle;
int pwmTrip;
int current_duty_cycle;
int rampCounter;
int accelDelay_ms_dc, accelDelay_old, accelDelay = 0;
int hall_state;
int hall_read;
EPWM_INFO epwm6_info;
EPWM_INFO epwm5_info;
EPWM_INFO epwm3_info;
bool direction_CCW = true, direction_old = false;
int StopMotor = false;
int motorBrakeType = 0;

float SenseResistorValue;
int CSAgain;
float OCLIM;
int FaultLED;
int PVDD_OV_Fault;
int PVDD_UV_Fault;
int OC_Fault;
int DRV8329_Fault;
int FaultRESET;
int FaultCounter;
int FaultLimit;

float PVDD_OVLO_LIM;
float PVDD_UVLO_LIM;
int VSENPVDD_ADCREAD;
float VSENPVDD_V;
int VSENA_ADCREAD;
float VSENA_V;
int VSENB_ADCREAD;
float VSENB_V;
int VSENC_ADCREAD;
float VSENC_V;
int ISEN_ADCREAD;
float ISEN_CURRENT;

int POT_ADCREAD;

int  OutputEnable;
bool PotEnable;
bool DRVOFF_signal = 0, DRVOFF_signal_old;
bool nSLEEP_signal = 0, nSLEEP_signal_old;

int temp, temp1 = 0;

//
// Function Prototypes
//
void resetDRV(void);
void DRV_InitGpio(void);
void DRV_InitGpioInput(void);
void DRV_InitGpioOutput(void);
void EPWM_Init(void);
void error (void);
void Set_HS0_LS1(EPWM_INFO *epwm_info);
void Set_HS0_LS0(EPWM_INFO *epwm_info);
void Set_PWM(EPWM_INFO *epwm_info, int duty_cycle);

void InitADCs(void);
void SetVREF(int module, int mode, int ref);
int16_t sampleADC_A(int channel);
int16_t sampleADC_B(int channel);
int16_t sampleADC_C(int channel);

void Config_evm_spi(void);
Uint16 spi_xmit(Uint16 spiFrame);
Uint16 spi_write(Uint16 addr, Uint16 data);
Uint16 spi_read(Uint16 addr);

void main(void)
{
    //
    // Initialize System Control:
    // PLL, WatchDog, enable Peripheral Clocks
    //
    InitSysCtrl();

    //
    // Initialize GPIO:
    // These GPIOs control LEDs, AFE GPIOS.
    //
    InitGpio();
    InitADCs();

    DRV_InitGpioInput();
    DRV_InitGpioOutput();

    // Disable and clear all CPU interrupts
    IER = 0x0000;
    IFR = 0x0000;

    //
    //initialize the ePWM
    //

    EALLOW;
    CpuSysRegs.PCLKCR0.bit.TBCLKSYNC = 0;
    EDIS;

    EPWM_Init();

    EALLOW;
    CpuSysRegs.PCLKCR0.bit.TBCLKSYNC = 1;
    EDIS;

    //Initialize and Setup DRV
    resetDRV();
    GPIO_WritePin(MCU_LED,0);    //set MCU LED low to turn on

    //Commutation Code and Settings
    pwmDutyCycle = 0.0;
    rampCounter = 0;
    pwmTrip = 0;
    accelDelay_ms_dc = 50;

    SenseResistorValue = 0.001;      //1mOhm sense resistor
    CSAgain = 5;                    //CSA gain of 20

    PVDD_OVLO_LIM = 80;              //PVDD input range 4.5V - 60V
    PVDD_UVLO_LIM = 2;

    OCLIM = 30;                      //30A Software Over-Current Limit

    FaultLED = 0;
    PVDD_OV_Fault = 0;
    PVDD_UV_Fault = 0;
    OC_Fault = 0;
    FaultRESET = 0;

    FaultCounter = 0;
    FaultLimit = 10;

    OutputEnable = 0;
    PotEnable = 0;

    while(1)
    {
        if(!OutputEnable)
        {
            pwmDutyCycle = 0;
            PotEnable = 0;
        }
        else if(OutputEnable == 1)
        {
            if(FaultLED)
            {
                OutputEnable = 0;
            }
        }

        VSENA_ADCREAD = sampleADC_A(5);       //ADCINA5
        VSENB_ADCREAD = sampleADC_B(0);      //ADCINB0
        VSENC_ADCREAD = sampleADC_C(2);      //ADCINC2
        VSENPVDD_ADCREAD = sampleADC_B(1);      //ADCINB1
        ISEN_ADCREAD = sampleADC_C(0);       //ADCINC0

        VSENPVDD_V = VSENPVDD_ADCREAD * 0.01857174495; //VSENPVDD
        VSENA_V = VSENA_ADCREAD * 0.01857174495;       //VSENA
        VSENB_V = VSENB_ADCREAD * 0.01857174495;       //VSENB
        VSENC_V = VSENC_ADCREAD * 0.01857174495;       //VSENC
        ISEN_CURRENT = (((float)(ISEN_ADCREAD) / 4095 * 3.3) - (3.29/7.92)) / SenseResistorValue / CSAgain;

        if(nFAULT_flag == 0 || VSENPVDD_V > PVDD_OVLO_LIM || VSENPVDD_V < PVDD_UVLO_LIM || (Uint16)(ISEN_CURRENT) > OCLIM)
        {

            FaultCounter++;

            if(FaultCounter >= FaultLimit)
            {
                FaultLED = 1;
                if(VSENPVDD_V > PVDD_OVLO_LIM)        //PVDD OV/UV
                {
                    PVDD_OV_Fault = 1;
                }

                else if(VSENPVDD_V < PVDD_UVLO_LIM)        //PVDD OV/UV
                {
                    PVDD_UV_Fault = 1;
                }

                else if((Uint16)(ISEN_CURRENT) > OCLIM)              //Over-Current
                {
                    OC_Fault = 1;
                }

                else if(nFAULT_flag == 0)
                {
                    DRV8329_Fault = 1;
                }

                OutputEnable = 0;
                FaultCounter = 0;
            }
        }
        else                                            //if no FAULT this cycle, reset the fault counter
        {
            FaultCounter = 0;
        }

        //Clear faults
        if(FaultRESET)                                 //clear all FAULT bits and FAULT LED
        {
            FaultLED = 0;
            PVDD_OV_Fault = 0;
            PVDD_UV_Fault = 0;
            DRV8329_Fault = 0;
            OC_Fault = 0;
            FaultRESET = 0;
            FaultCounter = 0;
            GPIO_WritePin(nSLEEP,0); //nSLEEP reset pulse = 20us
            for (temp=0;temp<500;temp++);
            GPIO_WritePin(nSLEEP,1); //nSLEEP reset pulse
        }

        if(nSLEEP_signal != nSLEEP_signal_old)
        {
            nSLEEP_signal_old = nSLEEP_signal;
            if (nSLEEP_signal)
            {
                GPIO_WritePin(nSLEEP,0);
            }
            else
            {
                GPIO_WritePin(nSLEEP,1);
            }
        }

        if(DRVOFF_signal != DRVOFF_signal_old)
        {
            DRVOFF_signal_old = DRVOFF_signal;
            if (DRVOFF_signal)
            {
                GPIO_WritePin(DRVOFF,1);
            }
            else
            {
                GPIO_WritePin(DRVOFF,0);
            }
        }

        if (PWM_FREQ != PWM_FREQ_old)
        {
            PWM_FREQ_old = PWM_FREQ;
            PWM_PERIOD = 48300000/PWM_FREQ;
            EPwm6Regs.TBPRD = PWM_PERIOD;
            EPwm5Regs.TBPRD = PWM_PERIOD;
            EPwm3Regs.TBPRD = PWM_PERIOD;
        }

        if (DEAD_TIME_ns != DEAD_TIME_old)
        {
            DEAD_TIME_old = DEAD_TIME_ns;
            EPwm6Regs.DBFED.bit.DBFED = DEAD_TIME_ns * 0.1;
            EPwm6Regs.DBRED.bit.DBRED = DEAD_TIME_ns * 0.1;
            EPwm5Regs.DBFED.bit.DBFED = DEAD_TIME_ns * 0.1;
            EPwm5Regs.DBRED.bit.DBRED = DEAD_TIME_ns * 0.1;
            EPwm3Regs.DBFED.bit.DBFED = DEAD_TIME_ns * 0.1;
            EPwm3Regs.DBRED.bit.DBRED = DEAD_TIME_ns * 0.1;
        }

        if (PotEnable) //duty cycle set directly thru SW/GUI
        {
            POT_ADCREAD = sampleADC_B(2);       //ADCINB2
            pwmDutyCycle = (float)(POT_ADCREAD)/40.95;
        }

        while (StopMotor)
        {
            if (motorBrakeType == 0) //Brake
            {
                Set_HS0_LS1(&epwm6_info);
                Set_HS0_LS1(&epwm5_info);
                Set_HS0_LS1(&epwm3_info);
            }
            if (motorBrakeType == 1) //Coast
            {
                Set_HS0_LS0(&epwm6_info);
                Set_HS0_LS0(&epwm5_info);
                Set_HS0_LS0(&epwm3_info);
            }
            current_duty_cycle = PWM_PERIOD;
            pwmTrip = 0;
        }

        if (direction_CCW != direction_old)
        {
            direction_old = direction_CCW;
            //Coast Motor for 1 second, reset PWM period value, ramp up in opposite direction to new duty cycle
            Set_HS0_LS0(&epwm6_info);
            Set_HS0_LS0(&epwm5_info);
            Set_HS0_LS0(&epwm3_info);
            for (temp=0;temp<3000;temp++){
                for (temp1=0;temp1<3000;temp1++);
            }
            current_duty_cycle = PWM_PERIOD;
            pwmTrip = 0;
        }

        if (accelDelay_ms_dc != accelDelay_old)
        {
            accelDelay_old = accelDelay_ms_dc;
            accelDelay = (Uint16)((float)(accelDelay_ms_dc)*2.381);
        }

        rampCounter++;

        if(rampCounter == accelDelay)
         {
           if(pwmTrip < (PWM_PERIOD * (pwmDutyCycle/100)))
           {
               pwmTrip = pwmTrip + 1;
           }
           else if(pwmTrip > (PWM_PERIOD * (pwmDutyCycle/100)))
           {
               pwmTrip = pwmTrip - 1;
           }
           rampCounter = 0;
         }

        current_duty_cycle = PWM_PERIOD - pwmTrip;

        //Check Hall Sensors
        hall_read = ((GpioDataRegs.GPBDAT.bit.GPIO58 << 2) | (GpioDataRegs.GPADAT.bit.GPIO23 << 1) | (GpioDataRegs.GPBDAT.bit.GPIO59 << 0));
        hall_state = (hall_read^0xF)&0x7;
        if (direction_CCW)
        {
            if (hall_state)
            {
                switch(hall_state)
                {
                case 1:
                    //B-C
                    Set_HS0_LS0(&epwm6_info);
                    Set_PWM(&epwm5_info, current_duty_cycle);
                    Set_HS0_LS1(&epwm3_info);
                    break;
                case 2:
                    //A-C
                    Set_PWM(&epwm6_info, current_duty_cycle);
                    Set_HS0_LS1(&epwm5_info);
                    Set_HS0_LS0(&epwm3_info);
                    break;
                case 3:
                    //A-B
                    Set_PWM(&epwm6_info, current_duty_cycle);
                    Set_HS0_LS0(&epwm5_info);
                    Set_HS0_LS1(&epwm3_info);
                    break;
                case 4:
                    //C-B
                    Set_HS0_LS1(&epwm6_info);
                    Set_HS0_LS0(&epwm5_info);
                    Set_PWM(&epwm3_info, current_duty_cycle);
                    break;
                case 5:
                    //C-A
                    Set_HS0_LS1(&epwm6_info);
                    Set_PWM(&epwm5_info, current_duty_cycle);
                    Set_HS0_LS0(&epwm3_info);
                    break;
                case 6:
                    //B-A
                    Set_HS0_LS0(&epwm6_info);
                    Set_HS0_LS1(&epwm5_info);
                    Set_PWM(&epwm3_info, current_duty_cycle);
                    break;
                } //end of switch
            } //end of if
        }
        else
        {
            if (hall_state)
            {
                switch(hall_state)
                {
                case 6:
                    //B-C
                    Set_HS0_LS0(&epwm6_info);
                    Set_PWM(&epwm5_info, current_duty_cycle);
                    Set_HS0_LS1(&epwm3_info);
                    break;
                case 5:
                    //A-C
                    Set_PWM(&epwm6_info, current_duty_cycle);
                    Set_HS0_LS1(&epwm5_info);
                    Set_HS0_LS0(&epwm3_info);
                    break;
                case 4:
                    //A-B
                    Set_PWM(&epwm6_info, current_duty_cycle);
                    Set_HS0_LS0(&epwm5_info);
                    Set_HS0_LS1(&epwm3_info);
                    break;
                case 3:
                    //C-B
                    Set_HS0_LS1(&epwm6_info);
                    Set_HS0_LS0(&epwm5_info);
                    Set_PWM(&epwm3_info, current_duty_cycle);
                    break;
                case 2:
                    //C-A
                    Set_HS0_LS1(&epwm6_info);
                    Set_PWM(&epwm5_info, current_duty_cycle);
                    Set_HS0_LS0(&epwm3_info);
                    break;
                case 1:
                    //B-A
                    Set_HS0_LS0(&epwm6_info);
                    Set_HS0_LS1(&epwm5_info);
                    Set_PWM(&epwm3_info, current_duty_cycle);
                    break;
                } //end of switch(hall_state)
            } //end of if(hall_state)
        } //end of if(direction)
    } //end of while
}

//
// DRV_InitGpio - Initialize the GPIOs on launchpad and boosterpack
//
void DRV_InitGpioInput()
{
    EALLOW; // below registers are "protected", allow access.
    // GPIO DRV Hall A
    GPIO_SetupPinMux(HALLA, GPIO_MUX_CPU1, 0);
    GPIO_SetupPinOptions(HALLA, GPIO_INPUT, GPIO_PULLUP);

    // GPIO DRV Hall B
    GPIO_SetupPinMux(HALLB, GPIO_MUX_CPU1, 0);
    GPIO_SetupPinOptions(HALLB, GPIO_INPUT, GPIO_PULLUP);

    // GPIO DRV Hall C
    GPIO_SetupPinMux(HALLC, GPIO_MUX_CPU1, 0);
    GPIO_SetupPinOptions(HALLC, GPIO_INPUT, GPIO_PULLUP);

    // nFAULT
    GPIO_SetupPinMux(nFAULT, GPIO_MUX_CPU1, 0);
    GPIO_SetupPinOptions(nFAULT, GPIO_INPUT, GPIO_PULLUP);

    EDIS;
    // Disable register access
}

void DRV_InitGpioOutput()
{
    EALLOW;

    //INHA DRV
    GpioCtrlRegs.GPADIR.bit.GPIO10 = 1;
    GpioCtrlRegs.GPAPUD.bit.GPIO10 = 1;    // Disable pull-up on GPIO10 (EPWM6A)
    GpioCtrlRegs.GPAMUX1.bit.GPIO10 = 1;   // Configure GPIO10 as EPWM6A
    //INLA DRV
    GpioCtrlRegs.GPADIR.bit.GPIO11 = 1;
    GpioCtrlRegs.GPAPUD.bit.GPIO11 = 1;    //...
    GpioCtrlRegs.GPAMUX1.bit.GPIO11 = 1;
    //INHB DRV
    GpioCtrlRegs.GPADIR.bit.GPIO8 = 1;
    GpioCtrlRegs.GPAPUD.bit.GPIO8 = 1;
    GpioCtrlRegs.GPAMUX1.bit.GPIO8 = 1;
    //INLB DRV
    GpioCtrlRegs.GPADIR.bit.GPIO9 = 1;
    GpioCtrlRegs.GPAPUD.bit.GPIO9 = 1;
    GpioCtrlRegs.GPAMUX1.bit.GPIO9 = 1;
    //INHC DRV
    GpioCtrlRegs.GPADIR.bit.GPIO4 = 1;
    GpioCtrlRegs.GPAPUD.bit.GPIO4 = 1;
    GpioCtrlRegs.GPAMUX1.bit.GPIO4 = 1;
    //INLC DRV
    GpioCtrlRegs.GPADIR.bit.GPIO5 = 1;
    GpioCtrlRegs.GPAPUD.bit.GPIO5 = 1;
    GpioCtrlRegs.GPAMUX1.bit.GPIO5 = 1;

    //MCU LED
    GPIO_SetupPinMux(MCU_LED, GPIO_MUX_CPU1, 0);
    GPIO_SetupPinOptions(MCU_LED, GPIO_OUTPUT, GPIO_PUSHPULL);

    //nSLEEP
    GPIO_SetupPinMux(nSLEEP, GPIO_MUX_CPU1, 0);
    GPIO_SetupPinOptions(nSLEEP, GPIO_OUTPUT, GPIO_PUSHPULL);
    GPIO_WritePin(nSLEEP,1);

    EDIS;
}

//
// EPWM_Init - Initialize EPWM configuration
//
void EPWM_Init()
{
    //
    // enable PWM6, PWM5 and PWM3
    //
    CpuSysRegs.PCLKCR2.bit.EPWM6=1;
    CpuSysRegs.PCLKCR2.bit.EPWM5=1;
    CpuSysRegs.PCLKCR2.bit.EPWM3=1;
    //
    // Setup TBCLK
    //
    EPwm6Regs.TBPRD = PWM_PERIOD;                // Set timer period 16000 TBCLKs
    EPwm6Regs.TBPHS.bit.TBPHS = 0x0000;          // Phase is 0
    EPwm6Regs.TBCTR = 0x0000;                    // Clear counter
    EPwm5Regs.TBPRD = PWM_PERIOD;                // Set timer period 16000 TBCLKs
    EPwm5Regs.TBPHS.bit.TBPHS = 0x0000;          // Phase is 0
    EPwm5Regs.TBCTR = 0x0000;                    // Clear counter
    EPwm3Regs.TBPRD = PWM_PERIOD;                // Set timer period 16000 TBCLKs
    EPwm3Regs.TBPHS.bit.TBPHS = 0x0000;          // Phase is 0
    EPwm3Regs.TBCTR = 0x0000;                    // Clear counter

    //
    // Setup counter mode
    //
    EPwm6Regs.TBCTL.bit.CTRMODE = TB_COUNT_UPDOWN;    // Count up/down
    EPwm6Regs.TBCTL.bit.HSPCLKDIV = TB_DIV1;      // Clock ratio to SYSCLKOUT
    EPwm6Regs.TBCTL.bit.CLKDIV = TB_DIV1;
    EPwm5Regs.TBCTL.bit.CTRMODE = TB_COUNT_UPDOWN;    // Count up/down
    EPwm5Regs.TBCTL.bit.HSPCLKDIV = TB_DIV1;      // Clock ratio to SYSCLKOUT
    EPwm5Regs.TBCTL.bit.CLKDIV = TB_DIV1;
    EPwm3Regs.TBCTL.bit.CTRMODE = TB_COUNT_UPDOWN;    // Count up/down
    EPwm3Regs.TBCTL.bit.HSPCLKDIV = TB_DIV1;      // Clock ratio to SYSCLKOUT
    EPwm3Regs.TBCTL.bit.CLKDIV = TB_DIV1;

    //
    // Setup shadowing
    //
    EPwm6Regs.CMPCTL.bit.SHDWAMODE = CC_SHADOW;
    EPwm6Regs.CMPCTL.bit.SHDWBMODE = CC_SHADOW;
    EPwm6Regs.CMPCTL.bit.LOADAMODE = CC_CTR_ZERO; // Load on Zero
    EPwm6Regs.CMPCTL.bit.LOADBMODE = CC_CTR_ZERO;
    EPwm5Regs.CMPCTL.bit.SHDWAMODE = CC_SHADOW;
    EPwm5Regs.CMPCTL.bit.SHDWBMODE = CC_SHADOW;
    EPwm5Regs.CMPCTL.bit.LOADAMODE = CC_CTR_ZERO; // Load on Zero
    EPwm5Regs.CMPCTL.bit.LOADBMODE = CC_CTR_ZERO;
    EPwm3Regs.CMPCTL.bit.SHDWAMODE = CC_SHADOW;
    EPwm3Regs.CMPCTL.bit.SHDWBMODE = CC_SHADOW;
    EPwm3Regs.CMPCTL.bit.LOADAMODE = CC_CTR_ZERO; // Load on Zero
    EPwm3Regs.CMPCTL.bit.LOADBMODE = CC_CTR_ZERO;

    EPwm6Regs.AQCTLA.bit.CAU = AQ_SET; // set actions for EPWM1A
    EPwm6Regs.AQCTLA.bit.CAD = AQ_CLEAR;
    EPwm6Regs.DBCTL.bit.OUT_MODE = DB_FULL_ENABLE; // enable Dead-band module
    EPwm6Regs.DBCTL.bit.POLSEL = DB_ACTV_HIC; // Active Hi complementary
    EPwm6Regs.DBFED.bit.DBFED = DEAD_TIME_ns * 0.1;
    EPwm6Regs.DBRED.bit.DBRED = DEAD_TIME_ns * 0.1;
    EPwm5Regs.AQCTLA.bit.CAU = AQ_SET; // set actions for EPWM1A
    EPwm5Regs.AQCTLA.bit.CAD = AQ_CLEAR;
    EPwm5Regs.DBCTL.bit.OUT_MODE = DB_FULL_ENABLE; // enable Dead-band module
    EPwm5Regs.DBCTL.bit.POLSEL = DB_ACTV_HIC; // Active Hi complementary
    EPwm5Regs.DBFED.bit.DBFED = DEAD_TIME_ns * 0.1;
    EPwm5Regs.DBRED.bit.DBRED = DEAD_TIME_ns * 0.1;
    EPwm3Regs.AQCTLA.bit.CAU = AQ_SET; // set actions for EPWM1A
    EPwm3Regs.AQCTLA.bit.CAD = AQ_CLEAR;
    EPwm3Regs.DBCTL.bit.OUT_MODE = DB_FULL_ENABLE; // enable Dead-band module
    EPwm3Regs.DBCTL.bit.POLSEL = DB_ACTV_HIC; // Active Hi complementary
    EPwm3Regs.DBFED.bit.DBFED = DEAD_TIME_ns * 0.1;
    EPwm3Regs.DBRED.bit.DBRED = DEAD_TIME_ns * 0.1;

    //
    //Disable Interrupt
    //
    EPwm6Regs.ETSEL.bit.INTEN = 0;                // disable INT
    EPwm5Regs.ETSEL.bit.INTEN = 0;                // disable INT
    EPwm3Regs.ETSEL.bit.INTEN = 0;                // disable INT


    //Set Handles
    epwm6_info.EPwmRegHandle = &EPwm6Regs;
    epwm5_info.EPwmRegHandle = &EPwm5Regs;
    epwm3_info.EPwmRegHandle = &EPwm3Regs;

    //Set Initial Compare values to be counter + 1 so that low side gates turn on and stay on to charge HS bootstrap
    EPwm6Regs.CMPA.bit.CMPA = PWM_PERIOD + 1;
    EPwm5Regs.CMPA.bit.CMPA = PWM_PERIOD + 1;
    EPwm3Regs.CMPA.bit.CMPA = PWM_PERIOD + 1;
}
void Set_HS0_LS1(EPWM_INFO *epwm_info)
{
    epwm_info->EPwmRegHandle->DBCTL.bit.POLSEL = DB_ACTV_HIC;    //set to keep low side on
    epwm_info->EPwmRegHandle->CMPA.bit.CMPA = (PWM_PERIOD + 1); // Set so PWM counter never trips, keeping high side off
}

void Set_HS0_LS0(EPWM_INFO *epwm_info)
{
    epwm_info->EPwmRegHandle->DBCTL.bit.POLSEL = DB_ACTV_HI;     //Set to keep low side off
    epwm_info->EPwmRegHandle->CMPA.bit.CMPA = (PWM_PERIOD + 1); // Set so PWM counter never trips, keeping high side off
}

void Set_PWM(EPWM_INFO *epwm_info, int duty_cycle)
{
    epwm_info->EPwmRegHandle->DBCTL.bit.POLSEL = DB_ACTV_HIC;
    epwm_info->EPwmRegHandle->CMPA.bit.CMPA = duty_cycle;
}

void InitADCs()
{
           //
           // Set the VREF to internal
           //
           SetVREF(ADC_ADCA, ADC_INTERNAL, ADC_VREF3P3);
           SetVREF(ADC_ADCB, ADC_INTERNAL, ADC_VREF3P3);
           SetVREF(ADC_ADCC, ADC_INTERNAL, ADC_VREF3P3);

           //
           // Configure the ADC: Initialize the ADC
           //
           EALLOW;

           //
           // write configurations
           //
           AdcaRegs.ADCCTL2.bit.PRESCALE = 6;      // set ADCCLK divider to /4
           AdcbRegs.ADCCTL2.bit.PRESCALE = 6;
           AdccRegs.ADCCTL2.bit.PRESCALE = 6;
           //
           // Set pulse positions to late
           //
           AdcaRegs.ADCCTL1.bit.INTPULSEPOS = 1;
           AdcbRegs.ADCCTL1.bit.INTPULSEPOS = 1;
           AdccRegs.ADCCTL1.bit.INTPULSEPOS = 1;

           //
           // power up the ADCs
           //
           AdcaRegs.ADCCTL1.bit.ADCPWDNZ = 1;
           AdcbRegs.ADCCTL1.bit.ADCPWDNZ = 1;
           AdccRegs.ADCCTL1.bit.ADCPWDNZ = 1;

           //
           // delay for 1ms to allow ADC time to power up
           //
           DELAY_US(1000);

           EALLOW;

}

void SetVREF(int module, int mode, int ref)
{
    Uint16 *offset, offval;

    //
    // Define offset locations from OTP
    //
    offset = (Uint16 *)(0x70594 + (module * 6));

    if((mode == ADC_INTERNAL) && (ref == ADC_VREF3P3))
    {
        offval = (*offset) >> 8;    // Internal / 1.65v mode offset
    }
    else
    {
        offval = (*offset) & 0xFF;  // All other modes
    }

    //
    // Write offset trim values and configure reference modes
    //
    EALLOW;
    switch(module)
    {
        case 0:
            AdcaRegs.ADCOFFTRIM.bit.OFFTRIM = offval;
            AnalogSubsysRegs.ANAREFCTL.bit.ANAREFASEL = mode;
            AnalogSubsysRegs.ANAREFCTL.bit.ANAREFA2P5SEL = ref;
            break;
        case 1:
            AdcbRegs.ADCOFFTRIM.bit.OFFTRIM = offval;
            AnalogSubsysRegs.ANAREFCTL.bit.ANAREFBSEL = mode;
            AnalogSubsysRegs.ANAREFCTL.bit.ANAREFB2P5SEL = ref;
            break;
        case 2:
            AdccRegs.ADCOFFTRIM.bit.OFFTRIM = offval;
            AnalogSubsysRegs.ANAREFCTL.bit.ANAREFCSEL = mode;
            AnalogSubsysRegs.ANAREFCTL.bit.ANAREFC2P5SEL = ref;
            break;
        default:
            break;
    }
    EDIS;
}

int16_t sampleADC_A(int channel)
{
    int16_t sample;

    EALLOW;

    AdcaRegs.ADCSOC0CTL.bit.CHSEL = channel;

    AdcaRegs.ADCSOC0CTL.bit.ACQPS = 25;
    AdcaRegs.ADCINTSEL1N2.bit.INT1SEL = 1;          //end of SOC1 will set INT1 flag
    AdcaRegs.ADCINTSEL1N2.bit.INT1E = 1;            //enable INT1 flag
    AdcaRegs.ADCINTFLGCLR.bit.ADCINT1 = 1;          //make sure INT1 flag is cleared

    AdcaRegs.ADCSOCFRC1.all = 0x03;                 // Force start of conversion on SOC0

    while(AdcaRegs.ADCINTFLG.bit.ADCINT1 == 0)      // Wait for end of conversion.
    {
         // Wait for ADCINT1
    }
    AdcaRegs.ADCINTFLGCLR.bit.ADCINT1 = 1;          // Clear ADCINT1

    sample = AdcaResultRegs.ADCRESULT0;             // Get ADC sample result from SOC0

    return(sample);
}

int16_t sampleADC_B(int channel)
{
    int16_t sample;

    EALLOW;

    AdcbRegs.ADCSOC0CTL.bit.CHSEL = channel;

    AdcbRegs.ADCSOC0CTL.bit.ACQPS = 25;
    AdcbRegs.ADCINTSEL1N2.bit.INT1SEL = 1;          //end of SOC1 will set INT1 flag
    AdcbRegs.ADCINTSEL1N2.bit.INT1E = 1;            //enable INT1 flag
    AdcbRegs.ADCINTFLGCLR.bit.ADCINT1 = 1;          //make sure INT1 flag is cleared

    AdcbRegs.ADCSOCFRC1.all = 0x03;                 // Force start of conversion on SOC0

    while(AdcbRegs.ADCINTFLG.bit.ADCINT1 == 0)      // Wait for end of conversion.
    {
         // Wait for ADCINT1
    }
    AdcbRegs.ADCINTFLGCLR.bit.ADCINT1 = 1;          // Clear ADCINT1

    sample = AdcbResultRegs.ADCRESULT0;             // Get ADC sample result from SOC0

    return(sample);
}

int16_t sampleADC_C(int channel)
{
    int16_t sample;

    EALLOW;

    AdccRegs.ADCSOC0CTL.bit.CHSEL = channel;
    AdccRegs.ADCSOC0CTL.bit.ACQPS = 25;
    AdccRegs.ADCINTSEL1N2.bit.INT1SEL = 1;  //end of SOC1 will set INT1 flag
    AdccRegs.ADCINTSEL1N2.bit.INT1E = 1;    //enable INT1 flag
    AdccRegs.ADCINTFLGCLR.bit.ADCINT1 = 1;  //make sure INT1 flag is cleared

    AdccRegs.ADCSOCFRC1.all = 0x03;                 // Force start of conversion on SOC0

    while(AdccRegs.ADCINTFLG.bit.ADCINT1 == 0)       // Wait for end of conversion.
    {
        // Wait for ADCINT1
    }
    AdccRegs.ADCINTFLGCLR.bit.ADCINT1 = 1;          // Clear ADCINT1

    sample = AdccResultRegs.ADCRESULT0;             // Get ADC sample result from SOC0

    return(sample);
}

void resetDRV(void){    //set nSLEEP low for 100us
    GPIO_WritePin(nSLEEP,0);
    int i;
    for(i=0;i<10000;i++);
    GPIO_WritePin(nSLEEP,1);
}

//
// error - Halt debugger when called
//
void error(void)
{
    ESTOP0;         // Stop here and handle error
}


//
// End of file
//

